import 'dart:async';
import 'dart:math';

import 'package:after_layout/after_layout.dart';
import 'package:flutter/widgets.dart';
import 'package:scroll_to_index/scroll_to_index.dart';

class DigitAnimation extends StatefulWidget {
  const DigitAnimation({
    super.key,
    required this.value,
    required this.style,
  });

  final int value;
  final TextStyle style;

  @override
  State<DigitAnimation> createState() => _DigitAnimationState();
}

class _DigitAnimationState extends State<DigitAnimation>
    with SingleTickerProviderStateMixin, AfterLayoutMixin {
  int _currentIndex = 0;
  late final AutoScrollController _autoScrollController;

  @override
  void initState() {
    super.initState();
    _autoScrollController = AutoScrollController();
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
  }

  @override
  void didUpdateWidget(covariant DigitAnimation oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (displayedNumber == widget.value) return;
    navigateToIndex(widget.value);
  }

  @override
  FutureOr<void> afterFirstLayout(BuildContext context) {
    navigateToIndex(widget.value);
  }

  int get displayedNumber => _currentIndex % 10;

  void navigateToIndex(int index) {
    if (_autoScrollController.isAutoScrolling) return;

    var diffrence = _currentIndex - index;

    if (diffrence.isNegative) {
      diffrence *= -1;
    }

    _autoScrollController.scrollToIndex(
      index,
      preferPosition: AutoScrollPosition.begin,
      duration: Duration(
        milliseconds: max(200, 200 * diffrence),
      ),
    );
    _currentIndex = index;
  }

  @override
  void dispose() {
    _autoScrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final size = _textSize(
      '0123456789',
      widget.style,
    );

    return ClipRRect(
      child: Stack(
        children: [
          SizedBox(
            width: size.width,
            height: size.height,
            child: ListView.builder(
              padding: EdgeInsets.zero,
              controller: _autoScrollController,
              itemCount: 10,
              itemBuilder: (context, index) => AutoScrollTag(
                index: index,
                controller: _autoScrollController,
                key: ValueKey(index),
                child: Text(
                  '$index',
                  style: widget.style,
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }
}

Size _textSize(String text, TextStyle style) {
  final TextPainter textPainter = TextPainter(
    text: TextSpan(text: text, style: style),
    maxLines: 1,
    textDirection: TextDirection.ltr,
  )..layout(minWidth: 0, maxWidth: double.infinity);
  return textPainter.size;
}
